/*
 * Cheops Next Generation
 * 
 * Mark Spencer <markster@marko.net>
 *
 * Copyright(C) 1999, Adtran, Inc.
 * 
 * Distributed under the terms of the GNU General Public License (GPL) Version 2
 *
 * General event management definitions
 *
 */

#ifndef _EVENT_H
#define _EVENT_H

#include "cheops-types.h"

typedef struct event_header {
	u32 len;		/* Length of event (including header) */
	u16 hlen;		/* Length of header and options (like proxy) */
	u16 type;		/* Type of event (see below) */
	u32 flags;		/* Event specific flags */
} event_hdr;

/* Max number of events */
#define MAX_EVENT 256

/* Max event size */
#define MAX_EVENT_SIZE 65536
#define BIT(n) (1<<n)
/*
 * Flags for use in all event headers
 */
#define FLAG_FORCE	BIT(0)		/* Do not use cached response */
#define FLAG_PROXY	BIT(1)		/* Use specified proxy for request or
					   specify that reply is coming from
					   given proxy */
#define FLAG_IGNORE	BIT(2)		/* Ignore message if unknown */

/* Event may be followed by additional information like a proxy
   host, and then one of the following bits of information. */

typedef struct event_proxy {
#define HDRTYPE_PROXY	        1
	u8	hdrtype;	/* Type of header option */
	u8	family; 	/* Family (e.g. AF_INET) */
	u16	len;		/* Length of address */
	/* Address data */
} proxy;

/* ============  Standard Event Requests ================= */

/* 
 * Discover a given network, given a start and end,
 * IP, both in network byte order.
 */ 
typedef struct discover_ipv4_event {
#define EVENT_DISCOVER_IPV4 	2
	u32	start;		/* Start address */
	u32	end;		/* End address */
	u32	is_name;
	u32	len;
	void *np;
} discover_ipv4_e;

/*
 * OS detection request
 */
typedef struct osscan_event {
#define EVENT_OS_SCAN           3
	u32     length;
	u32     num_options;
	u32     ip;
	void	*np;
	//The first option starts after this
} os_scan_e;

/*
 * DNS Query event
 */
typedef struct dns_query_event {
#define EVENT_DNS_QUERY         5
	u32     length;
	u32     num_options;
	char    name[256];
	//The first option starts after this
} dns_query_e;

/*
 * Set the settings that the agent depends on
 */
typedef struct set_settings {
#define EVENT_SET_SETTINGS      4
	u32	flags;
	u32	discover_retries;	/* number discover of retries */
} set_settings_e; 

/* flags for the set_settings event */
#define SET_DISCOVER_RETRIES	0x0001
#define MAX_ALLOWABLE_DISCOVER_RETRIES 5

/* ============  Standard Event Replies ================= */

/*
 * Report an error string and code.
 * the code # is standard, and the 
 * string is some meaningful message.
 *
 * See cerror.h for more.
 */
typedef struct error_reply {
#define REPLY_ERROR             101
	u16 error;
	u16 len;
} error_r;

/*
 * Notification that a given IP address has
 * been discovered
 */
typedef struct discover_ipv4_reply {
#define REPLY_DISCOVER_IPV4 	102
	u32 ipaddr;
	void *np;
} discover_ipv4_r;

/*
 * OS detection request
 */
typedef struct osscan_reply {
#define REPLY_OS_SCAN           103
	u32     num_options;
	u32     ip;
	void	*np;
	//The first option starts after this
}os_scan_r;


/*
 * DNS Query request
 */
typedef struct dns_query_reply {
#define REPLY_DNS_QUERY         105
	u32     length;
	u32     num_options;
	u32     ip;
	char    name[256];
	//The first option starts after this
} dns_query_r;



typedef union {
	os_scan_e os_scan_e;
	os_scan_r os_scan_r;
	discover_ipv4_e discover_ipv4_e;
	discover_ipv4_r discover_ipv4_r;
	set_settings_e set_settings_e;
	dns_query_e dns_query_e;
	dns_query_r dns_query_r;
	error_r error_r;
} event;


/* An agent is someone who communicates to us over a fd of some
   sort.   */

#define AGENT_TYPE_LOCAL 1
#define AGENT_TYPE_IPV4 2

struct agent_struct;

typedef struct agent_struct agent;

struct agent_struct {
	agent *next;
	int type;	/* Kind of agent */
	int *id;		/* I/O ID */
	int s;		/* Socket */
	void *more; /* Anything else */
};

/*
 * A Cheops event handler takes it, a header, an event, and the source 
 * agent, and handles it.
 */

typedef int (*cheops_event_handler)(event_hdr *h, event *e, agent *a);
#define CHEOPS_EVENT_HANDLER(a) ((cheops_event_handler)(a))

/*
 * Request an agent to a specific remote host.
 */
extern agent *event_request_agent(int agent_type, void *addr, int is_default);



/*
 * Build a local listening agent.  Note that agent_type can be more
 * than one agent type, OR'd together.
 */
extern int event_create_agent(int agent_type);

/*
 * Send an event to an agent
 */
extern int event_send(agent *a, event_hdr *h);

/*
 * Destroy agent
 */
extern void event_destroy_agent(agent *a);

/*
 * Register a handler for an event type
 */

extern int event_register_handler(short type, cheops_event_handler h, int replace);

#endif
